%  control_arb.m
%  Kap. 5.2  
%  Arbeiten mit LTI-Modellen
%   - EIGENSCHAFTEN SETZEN UND NDERN
%   - DATEN ABRUFEN
%   - RANGFOLGE DER LTI-MODELLE UND VERERBUNG (nicht im Buch)
%   - ARITHMETISCHE OPERATIONEN
%   - AUSWHLEN, VERNDERN UND VERKNPFEN VON LTI-MODELLEN
%       * Extrahieren
%       * Zuweisen
%       * Lschen von Eingang 1
%       * Hinzufgen neuer Eingang
%       * Rckgekoppelter Integrator
%   - SPEZIELLE MODELLE
%   - UMWANDLUNG ZWISCHEN KONTINUIERLICHEN UND ZEITDISKRETEN SYSTEMEN

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Einstellungen

clear all
format compact
echo off
pause off
figure_defaults     % Default-Einstellungen fr Figuren
druck = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Schalter zum getrennten abarbeiten
if exist('alle') == 0 , alle = 1 ; end

if exist('eigen')    == 0 , eigen    = 0 + alle ; end
if exist('daten')    == 0 , daten    = 0 + alle ; end       
if exist('rang')     == 0 , rang     = 0 + alle ; end
if exist('arithm')   == 0 , arithm   = 0 + alle ; end
if exist('exmod')    == 0 , exmod    = 0 + alle ; end
if exist('spez')     == 0 , spez     = 0 + alle ; end
if exist('kontdisk') == 0 , kontdisk = 0 + alle ; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   - EIGENSCHAFTEN SETZEN UND NDERN
if eigen == 1 ,
    echo on
    sys = tf([1 -0.5],[1 1 -2],-1,'Notes','LTI-Objekt')
    pause
    set(sys,'Ts',0.05,'InputName','Eingang')
    pause
    sys.OutputName = 'Ausgang'
    pause
    abtastzeit = sys.Ts
    pause
    notiz = get(sys,'Notes') 
    pause
    get(sys)
    echo off
end % if eigen == 1 ,

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   - DATEN ABRUFEN
if daten == 1 ,
    echo on
    h = tf([2 -3],[1 1]) ;

    pause
    [num,den,Ts] = tfdata (h)
    num{1,1}                        % Abfrage des Cell--Arrays

    pause
    [num,den,Ts] = tfdata (h,'v'); 
    num
    echo off
end % daten == 1 ,


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   - RANGFOLGE DER LTI-MODELLE UND VERERBUNG (nicht im Buch)
if rang == 1 ,
    echo on 
    systf = tf([2 -3],[1 1]);
    sysss = ss([ 1 2 ; 3 4 ],[ 1 ; 0 ],[ 0 1 ],0);
  
    pause
    sys = systf + tf(sysss)           % Erst umwandeln, dann rechnen
    sys = tf(systf + sysss)           % Erst rechnen, dann umwandeln
    
    pause
    sysdtf = tf([2 -3],[1 1],-1,'Notes','Notiz')
    sysdss = ss([ 1 2 ; 3 4 ],[ 1 ; 0 ],[ 0 1 ],0,0.05,'Notes','Notiz')
  
    pause
    sysd = sysdtf + tf(sysdss)                                         
    sysd.Notes
    echo off 
end % if rang == 1 ,

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   - ARITHMETISCHE OPERATIONEN (nicht im Buch)
if arithm == 1 ,
    echo on
    sys1 = ss([ 1 2 ; 3 4 ],[ 1 1 ; 0 1 ],[ 3 1 ],0) ;
    sys2 = ss([ 4 3 ; 2 1 ],[ 2 3 ; 1 4 ],[ 0 1 ],0) ;
    
    pause
    % Addieren und Subtrahieren
    sys = sys1 + sys2 ;
%    sys = sys1 - sys2 ;
                           
    echo on
    sys1.a , sys2.a , sys.a  
    sys1.b , sys2.b , sys.b  
    sys1.c , sys2.c , sys.c  
    sys1.d , sys2.d , sys.d  
    echo off
    pause
end % if arithm == 1 ,

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   - AUSWHLEN, VERNDERN UNDVERKNPFEN VON LTI-MODELLEN
if exmod == 1 ,
    echo on
    h11 = tf([2 1],[1 0.1]);       % Eingang 1, Ausgang 1
    h12 = tf(1,[1 0.5]);           % Eingang 2, Ausgang 1
    h21 = tf(0,1);                 % Eingang 1, Ausgang 2
    h22 = tf(1,[1 -1]);            % Eingang 2, Ausgang 2
  
    systf = tf([h11 h12;h21 h22]) 
  
    pause
    % Extrahieren
    teilsys = systf(1,2)            % H von Eingang 2 nach Ausgang 1
  
    pause
    % Zuweisen
    teilsys = tf(1,[1 0])
    systf(1,2) = teilsys

    pause
    % Lschen von Eingang 1
    systf(:,1) = []

    pause
    % Hinzufgen neuer Eingang 
    systf(:,2) = [ tf(1,[1 -1]) ; tf(1,1) ]
    echo off
    pause

    % Rckgekoppelter Integrator
    integrator   = tf(1,[3 0])    
    proportional = tf(2,1)    
    
    feedback(integrator,proportional)
    echo off
    pause
end % if exmod == 1 ,

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   - SPEZIELLE MODELLE (nicht im Buch)
if spez == 1 ,
    echo on
    % Zufallswertsysteme
    sysr  = rss(2,3) 
    [num,den] = drmodel(2)
    
    pause
    % System 2. Ordnung
    [num,den] = ord2(10,sqrt(2))
    step(num,den)   
    echo off
    pause
end % if spez == 1 ,

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   - UMWANDLUNG ZWISCHEN KONTINUIERLICHEN UND ZEITDISKRETEN SYSTEMEN
if kontdisk == 1 ,
    echo on
    pause
    sysc  = tf([1],[1 .7 1])            % PT_2
    sysd  = c2d (sysc,1.5)               % Halteglied 0.~Ordnung
    sysd1 = c2d (sysc,1.5,'foh')         % Halteglied 1.~Ordnung
    sysd2 = c2d (sysc,1.5,'tustin')      % Tustin--Approximation
             
    figure
    step(sysc,'r-',sysd,'c--',sysd1,'g-.',sysd2,'k-',20)
    legend('sysc','sysd (ZOH)','sysd1 (FOH)','sysd2 (Tustin)',4)
%    set(get(gca,'Title'),'FontSize',14)
%    set(gcf,'PaperPosition',[1 1 20 9])
    if exist('druck') == 1 , print('-depsc','bsp_control_arb1.eps') , end
    if exist('druck') == 1 , print('-deps' ,'bsp_control_arb1_sw.eps') , end

    pause
    % Diskretisierung verzgerter Systeme
    sysc  = tf(1,[1 1])                % PT_1
    sysd  = c2d (sysc,2)               % Halteglied 0.~Ordnung
    sysdd = d2d (sysd,0.7)             % Neue Abtastzeit

    figure
    step(sysc,'r-',sysd,'c--',sysdd,'g-.',10)
    legend('sysc','sysd (ZOH)','sysdd  (ZOH resampled)',4)
%    set(get(gca,'Title'),'FontSize',14)
%    set(gcf,'PaperPosition',[1 1 20 9])
    if exist('druck') == 1 , print('-depsc','bsp_control_arb2.eps') , end
    if exist('druck') == 1 , print('-deps' ,'bsp_control_arb2_sw.eps') , end
    
    echo off
end % if kontdisk == 1 ,
